### Nested Logit model estimation

### Create the data array from the existing dataset
Data <- read.csv("C:/Users/yw543_000/Desktop/Data_Clean_English.csv",header=TRUE)
Data2 <- subset(Data, Data$ODDistanceM>=1000 & Data$ODDistanceM<=1500)
## Count the number of Data2 rows
hh <- nrow(Data2)

## Set the initial values of the parameters (the number in parenthesis corresponds to the number of parameters to be estimated)
b0 <- numeric(9) 

##### Define the log-likelihood function of the logit model#####

fr <- function(x) {
  ### declare the parameters###
  ## Alternative specific constants
  b1 <- x[1]
  b2 <- x[2]
  b3 <- x[3]
  b4 <- x[4]
  
  ##Time Variables
  d1 <- x[5]  #modetime
  d2 <- x[6]  #totalaccess
  
  ##Fare
  f1 <- x[7]
  
  ##Leisure
  f2 <- x[8]
  
  ##Scale parameter
  pa <- x[9] 
  
  ## declare the log-likelihood variable, set value to 0
  LL = 0
  
  ### For this choice problem, we consider the following 5 modes
  ## (train)- (public)
  ## (bus)  - (public)
  ## (car)  - (private)
  ## (bike) - (nomotor)
  ## (walk) - (nomotor)
  
  ## calculate the utility function: :introduce the desired explanatory variables in the function
  #modetime               #leisure dummy     #accesstime                                          # fare               # constant
  train  <- Data2$ModeAvailableTrain*exp(d1*Data2$TimeTrain/100 + f2*Data2$Leisure + d2*(Data2$AccessTimeTrain +Data2$EgressTimeTrain)/100 + f1*Data2$FareTrain/1000 + b1*matrix(1,nrow =hh,ncol=1))
  bus    <- Data2$ModeAvailableBus  *exp(d1*Data2$TimeBus  /100 + f2*Data2$Leisure + d2*(Data2$AccessTimeBus   +Data2$EgressTimeBus)  /100 + f1*Data2$FareBus  /1000 + b2*matrix(1,nrow =hh,ncol=1))
  car    <- Data2$ModeAvailableCar  *exp(d1*Data2$TimeCar  /100 + f2*Data2$Leisure                                                                                + b3*matrix(1,nrow =hh,ncol=1))
  bike   <- Data2$ModeAvailableBike *exp(d1*Data2$TimeBike /100 + f2*Data2$Leisure                                                                                + b4*matrix(1,nrow =hh,ncol=1))
  walk   <- Data2$ModeAvailableWalk *exp(d1*Data2$TimeWalk /100 + f2*Data2$Leisure                                                    )
  
  ### Construct nests.First, calculate public or private choice probabilityP(LV1)
  ## setting of log-sum variables
  logsum.public  <- log( ( (train + bus)!=0) * (train + bus) + ((train + bus)==0))
  logsum.private  <- log( ( car!=0) * car + (car==0))
  logsum.nomotor <- log( ( (bike + walk)!=0) * (bike + walk) + ((bike + walk)==0))
  
  ##Calculate public or private choice probability with these log-sum variables
  nume.public   <- (logsum.public  != 0) * exp(pa*logsum.public)  + (logsum.public  == 0)
  nume.private  <- (logsum.private != 0) * exp(pa*logsum.private) + (logsum.private == 0)
  nume.nomotor  <- (logsum.nomotor != 0) * exp(pa*logsum.nomotor) + (logsum.nomotor == 0)
  
  deno <- nume.public + nume.private + nume.nomotor
  P.public  <- nume.public  / deno
  P.private <- nume.private / deno
  P.nomotor <- nume.nomotor / deno
  
  ### Next, calculate conditional probability.P(LV2|LV1)
  ## conditional probability under public choice
  deno.public     <- train + bus
  P.train.public  <- train / ((deno.public!=0)*deno.public   + (deno.public  ==0))
  P.bus.public    <- bus   / ((deno.public!=0)*deno.public   + (deno.public  ==0))
  
  deno.private    <- car
  P.car.private   <- car   / ((deno.private!=0)*deno.private + (deno.private ==0))
  
  ## conditional probability under nomotor choice
  deno.nomotor    <- bike + walk
  P.bike.nomotor  <- bike  / ((deno.nomotor!=0)*deno.nomotor + (deno.nomotor ==0))
  P.walk.nomotor  <- walk  / ((deno.nomotor!=0)*deno.nomotor + (deno.nomotor ==0))
  
  ### Finally, calculate joint probability. P(LV1, LV2) = P(LV2|LV1)*P(LV1)
  P.train <- P.train.public * P.public
  P.bus   <- P.bus.public   * P.public
  P.car   <- P.car.private  * P.private
  P.bike  <- P.bike.nomotor * P.nomotor
  P.walk  <- P.walk.nomotor * P.nomotor
  
  ## Avoid problems stemming from choice probabilities becoming zero.
  P.train <- (P.train!=0)*P.train + (P.train==0)
  P.bus   <- (P.bus!=0)  *P.bus   + (P.bus==0)
  P.car   <- (P.car!=0)  *P.car   + (P.car==0)
  P.bike  <- (P.bike!=0) *P.bike  + (P.bike==0)
  P.walk  <- (P.walk!=0) *P.walk  + (P.walk==0)
  
  ## Choice results
  Ctrain   <- Data2$MainModeENG =="Rail"
  Cbus     <- Data2$MainModeENG =="Bus"
  Ccar     <- Data2$MainModeENG =="Car"
  Cbike    <- Data2$MainModeENG =="Bicycle"
  Cwalk    <- Data2$MainModeENG =="Walk"
  
  ## Calculate the Log-likelihood function
  LL <- colSums(Ctrain*log(P.train) + Cbus*log(P.bus) + Ccar  *log(P.car)   + Cbike  *log(P.bike) +Cwalk *log(P.walk))
  
}

##### Maximize the Log-likelihood function#####

##Parameter optimization 
res <- optim(b0,fr, method = "BFGS", hessian = TRUE, control=list(fnscale=-1))

## Parmeter estimation Hessian matrix calculation 
b   <- res$par
hhh <- res$hessian

## Calculate the t-statistic
tval <- b/sqrt(-diag(solve(hhh)))

## L(0), Log-Likelihood when all parameters are 0
L0 <- fr(b0)
## LL, maximium likelihood
LL <- res$value

##### Output #####
print(res)
## L(0)
print(L0)
## LL
print(LL)
##rho-square
print((L0-LL)/L0)
## adjusted rho-square
print((L0-(LL-length(b)))/L0)
##estimated parameter values
print(b)
## t-statistic 
print(tval)
